/*
tab-io.c
andrew snowden

history:
01/23/07 - created
*/

#include <stdio.h>
#include <malloc.h>
#include <memory.h>
#include <stdlib.h>
#include "avltree.h"
#include "tableaux.h"


static int read_status=0;
static char lnbuf[1024];


void print_tableau(tableau_t *tableau, FILE *outf) {
  char *fmt;
  int i, r;

  if (tableau->sides<10) fmt="%d";
  else fmt="%2d";
  for (r=0; r<2; r++) {
    for (i=0; i<tableau->width; i++) {
      fprintf(outf, fmt, tableau->row[r][i]);
      if (i<tableau->width-1) fputc(' ', outf);
    }
    fputc('\n', outf);
  }
}

void write_tableau(tableau_t *tableau, FILE *outf) {
  int i;

  fprintf(outf, "%d, %d; ", tableau->sides, tableau->weight);
  for (i=0; i<tableau->width; i++) {
    fprintf(outf, "%d", (int)tableau->row[0][i]);
    if (i<tableau->width-1) fprintf(outf, ", ");
    else fprintf(outf, "; ");
  }
  for (i=0; i<tableau->width; i++) {
    fprintf(outf, "%d", (int)tableau->row[1][i]);
    if (i<tableau->width-1) fprintf(outf, ", ");
    else fprintf(outf, ";\n");
  }
}

tableau_t *read_tableau(FILE *inf) {
  char *cp, *cp2, *cp3;
  int sides, weight, width;
  int i, x;
  tableau_t *tableau=NULL;

  if (!fgets(lnbuf, sizeof(lnbuf), inf)) {
    read_status=1;
    return NULL;
  }
  for (cp=lnbuf; isspace(*cp) && *cp; cp++);
  if (*cp=='\0') {
    read_status=0;
    return NULL;
  }
  cp=strchr(lnbuf, ';');
  if (!cp) goto error;
  *cp++='\0';
  cp2=strchr(lnbuf, ',');
  if (!cp2) goto error;
  *cp2++='\0';
  sides=atoi(lnbuf);
  weight=atoi(cp2);
  if (sides<2 || sides>=MAXSIDES) goto error;
  if (weight<1 || weight>=MAXWEIGHT) goto error;
  cp2=strchr(cp, ';');
  if (!cp2) goto error;
  *cp2++='\0';

  tableau=tableau_create_empty(sides, weight);
  width=tableau->width;

  for (i=0; i<width; i++) {
    if (!cp) goto error;
    cp3=strchr(cp, ',');
    if (cp3) *cp3++='\0';
    x=atoi(cp);
    if (x<1 || x>sides) goto error;
    tableau->row[0][i]=(unsigned char)x;
    cp=cp3;
  }

  cp=cp2;
  for (i=0; i<width; i++) {
    if (!cp) goto error;
    cp3=strchr(cp, ',');
    if (cp3) *cp3++='\0';
    x=atoi(cp);
    if (x<1 || x>sides) goto error;
    tableau->row[1][i]=(unsigned char)x;
    cp=cp3;
  }

  if (!tableau_is_valid(tableau)) goto error;

  return tableau;

error:
  if (tableau) tableau_free(tableau);
  read_status=-1;
  return NULL;
}


/*
status codes:
 0 - no status
 1 - end of file
-1 - error
*/
int read_tableau_status() {
  return read_status;
}

void write_sum(sum_t *sum, FILE *outf) {
  int i, x;

  for (i=0; i<sum->len; i++) {
    x=sum->count[i];
    if (i==0) {
      fprintf(outf, "%d", x);
    } else if (x>0) {
      fprintf(outf, "+%d", x);
    } else {
      fprintf(outf, "-%d", -x);
    }
    fprintf(outf, "[%d]", sum->id[i]);
  }
  fprintf(outf, ";\n");
}

sum_t *read_sum(FILE *inf) {
  sum_t *sum;
  char *cp, *cp2;
  int error=0;
  int count, id;

  if (!fgets(lnbuf, sizeof(lnbuf), inf)) return 0;
  cp=strchr(lnbuf, ';');
  if (!cp) return 0;
  *cp='\0';

  sum=sum_create();
  cp=lnbuf;

  while (1) {
    while (isspace(*cp) && *cp!='\0') cp++;
    if (*cp=='\0') break;
    cp2=cp;
    while (*cp2!='[' && *cp2!='\0') cp2++;
    if (*cp2!='[') {
      error=1;
      break;
    }
    *cp2++='\0';
    count=atoi(cp);
    cp=cp2;
    while (*cp2!=']' && *cp2!='\0') cp2++;
    if (*cp2!=']') {
      error=1;
      break;
    }
    *cp2++='\0';
    id=atoi(cp);
    cp=cp2;
    sum_add(sum, id, count);
  }

  if (error) {
    sum_free(sum);
    sum=NULL;
  }

  return sum;
}

