/*
tab-list-std.c
andrew snowden

history:
12/30/04 - created tableaux.c from older tableaux.c file
01/23/07 - split off tab-list-std.c
*/

#include <malloc.h>
#include <memory.h>
#include <stdlib.h>
#include "avltree.h"
#include "tableaux.h"


typedef struct {
  db_t *db;
  int width, sides, weight;
  tableau_t *tableau;
  unsigned char *counter, *left;
} std_data_t;


static int standard_gen_tableau(std_data_t *std) {
  int i, low=0;

  memcpy(std->tableau->row[0], std->counter, std->width);
  std->left[0]=0;
  for (i=1; i<=std->sides; i++)
    std->left[i]=std->weight;
  for (i=0; i<std->width; i++)
    std->left[std->counter[i]]--;
  for (i=0; i<std->width; i++) {
    while (std->left[low]==0) low++;
    if (low<=std->tableau->row[0][i]) return 0;
    std->tableau->row[1][i]=low;
    std->left[low]--;
  }
  return 1;
}

static int standard_init(std_data_t *std, db_t *db, int weight) {
  int i;

  std->db=db;
  std->sides=db->sides;
  std->weight=weight;
  std->width=std->weight*(std->sides/2);
  std->tableau=tableau_create_empty(db->sides, weight);
  if (!std->tableau) return 0;
  std->counter=(unsigned char *)malloc(std->width);
  if (!std->counter) {
    tableau_free(std->tableau);
    return 0;
  }
  std->left=(unsigned char *)malloc(std->sides+1);
  if (!std->left) {
    tableau_free(std->tableau);
    free(std->counter);
    return 0;
  }
  for (i=0; i<std->width; i++)
    std->counter[i]=i/std->weight+1;
  standard_gen_tableau(std);
  return 1;
}

static int standard_step_counter(std_data_t *std) {
  int i, n, cnt;

  do {
    for (i=0; i<std->width; i++)
      if (std->counter[std->width-i-1]<std->sides-1-i/std->weight) break;
    if (i>=std->width-std->weight) return 0;
    n=std->counter[std->width-i-1]+1;
    cnt=0;
    for (; i>=0; i--) {
      std->counter[std->width-i-1]=n;
      cnt++;
      if (cnt==std->weight) {
        n++;
        cnt=0;
      }
    }
  } while (!standard_gen_tableau(std));
  return 1;
}

static void standard_cleanup(std_data_t *std) {
  tableau_free(std->tableau);
  free(std->counter);
  free(std->left);
}

static void standard_create_lookup(db_t *db, int weight) {
  entry_t **ent;
  avl_it_t it;
  int i;

  ent=(entry_t **)malloc(sizeof(entry_t *)*db->stdcnt[weight-1]);
  if (!ent) return;
  avl_it_init(&it, db->avl[weight-1]);
  avl_it_first(&it);
  i=0;
  while (!avl_it_end(&it)) {
    ent[i++]=(entry_t *)avl_it_data(&it);
    avl_it_next(&it);
  }
  db->lookup[weight-1]=ent;
}

int standard_add_all(db_t *db, int weight, int wantlookup) {
  std_data_t std;
  int n=0;

  if (weight<1 || weight>MAXWEIGHT) return 0;
  if (db->avl[weight-1]!=NULL) return 0;

  if (!standard_init(&std, db, weight)) return 0;
  do {
    if (!db_add(db, std.tableau)) {
      standard_cleanup(&std);
      return 0;
    }
    n++;
  } while (standard_step_counter(&std));
  standard_cleanup(&std);
  db->stdcnt[weight-1]=n;
  if (wantlookup) standard_create_lookup(db, weight);
  return 1;
}

