/*
tab-stdize.c
andrew snowden

history:
12/30/04 - created tableaux.c from older tableaux.c file
01/23/07 - split off tab-stdize.c
*/

#include <malloc.h>
#include <memory.h>
#include <stdlib.h>
#include "avltree.h"
#include "tableaux.h"


static int tableau_col_cmp(const void *xxx1, const void *xxx2) {
  const unsigned char *x1=(const unsigned char *)xxx1;
  const unsigned char *x2=(const unsigned char *)xxx2;

  if (x1[0]<x2[0]) return -1;
  if (x1[0]>x2[0]) return 1;
  if (x1[1]<x2[1]) return -1;
  if (x1[1]>x2[1]) return 1;
  return 0;
}

/* puts in lexicographic order */
int tableau_reorder(tableau_t *tableau) {
  unsigned char *tmp;
  int i;

  tmp=(unsigned char*)malloc(2*tableau->width);
  if (!tmp) return 0;
  for (i=0; i<tableau->width; i++) {
    tmp[2*i]=tableau->row[0][i];
    tmp[2*i+1]=tableau->row[1][i];
  }
  qsort(tmp, tableau->width, 2, tableau_col_cmp);
  for (i=0; i<tableau->width; i++) {
    tableau->row[0][i]=tmp[2*i];
    tableau->row[1][i]=tmp[2*i+1];
  }
  free(tmp);
  return 1;
}

/* makes columns increasing.  returns -1 if odd number of flips
   required, 1 otherwise */
int tableau_sort_cols(tableau_t *tableau) {
  int i, x, s=1;

  for (i=0; i<tableau->width; i++) {
    if (tableau->row[0][i]>tableau->row[1][i]) {
      x=tableau->row[0][i];
      tableau->row[0][i]=tableau->row[1][i];
      tableau->row[1][i]=x;
      s*=-1;
    }
  }
  return s;
}

static int standardize_auxillary(db_t *db, tableau_t *tableau, sum_t *sum, int scale) {
  tableau_t *scratch;
  int i=0, n;

  if (!tableau_reorder(tableau)) return 0;

  for (i=0; i<tableau->width-1; i++)
    if (tableau->row[1][i]>tableau->row[1][i+1]) break;

  if (i==tableau->width-1) {
    entry_t *p;

    p=db_find_by_tableau(db, tableau);
    if (!p) return 0;
    return sum_add(sum, p->id, scale);
  }

  scratch=tableau_copy(tableau);
  if (!scratch) return 0;

  n=tableau->row[1][i];
  tableau->row[1][i]=tableau->row[1][i+1];
  tableau->row[1][i+1]=n;

  n=scratch->row[1][i];
  scratch->row[1][i]=scratch->row[0][i+1];
  scratch->row[0][i+1]=scratch->row[1][i+1];
  scratch->row[1][i+1]=n;

  n=standardize_auxillary(db, tableau, sum, scale);
  if (n)
    n=standardize_auxillary(db, scratch, sum, -scale);

  tableau_free(scratch);
  return n;
}

sum_t *tableau_standardize(db_t *db, tableau_t *tableau) {
  tableau_t *tmp;
  sum_t *sum;
  int s;

  tmp=tableau_copy(tableau);
  if (!tmp) return NULL;
  sum=sum_create();
  if (!sum) {
    tableau_free(tmp);
    return NULL;
  }
  s=tableau_sort_cols(tmp);
  if (!standardize_auxillary(db, tmp, sum, s)) {
    sum_free(sum);
    sum=NULL;
  } else
    sum_simplify(sum);
  free(tmp);
  return sum;
}

