/*
tableaux.h
12/30/04
andrew snowden

taken from old tableaux.c file
*/

#ifndef TABLEAUX_H
#define TABLEAUX_H

#include <stdio.h>
#include "avltree.h"

#define MAXSIDES  100
#define MAXWEIGHT 10

typedef struct {
  int sides, weight, width;
  unsigned char *row[2];
} tableau_t;


/*** SUM FUNCTIONS ***/


typedef struct {
  int len, alloc;
  int *count, *id;
} sum_t;

sum_t *sum_create();
void sum_free(sum_t *sum);
int sum_add(sum_t *sum, int id, int count);
void sum_simplify(sum_t *sum);


/*** POLYNOMIAL FUNCTIONS ***/


typedef struct {
  int degree, nterms;
  int *coefficient;
  int *monomial;
} poly_t;

poly_t *poly_read(FILE *inf);
void poly_free(poly_t *poly);


/*** DATABASE FUNCTIONS ***/


typedef struct tag_entry_t entry_t;

struct tag_entry_t {
  unsigned char *tableau[2];
  char wt;
  int id;
  entry_t *prev, *next;
};

typedef struct {
  int sides;
  avl_t *avl[MAXWEIGHT];
  int maxid[MAXWEIGHT];
  entry_t **lookup[MAXWEIGHT];
  int stdcnt[MAXWEIGHT];
} db_t;

entry_t *db_find_by_tableau(db_t *db, tableau_t *tableau);
entry_t *db_find_by_id(db_t *db, int weight, int id);
entry_t *db_lookup_std(db_t *db, int weight, int id);
int db_add(db_t *db, tableau_t *tableau);
void db_free(db_t *db);
db_t *db_create(int sides);
void db_print_entry(int sides, entry_t *p, FILE *outf);
void db_print_all(db_t *db, int weight, FILE *outf);
void db_write_all(db_t *db, int weight, FILE *outf);


/*** I/O FUNCTIONS ***/


void write_tableau(tableau_t *tableau, FILE *outf);
tableau_t *read_tableau(FILE *inf);
void print_tableau(tableau_t *tableau, FILE *outf);
int read_tableau_status();
void write_sum(sum_t *sum, FILE *outf);
sum_t *read_sum(FILE *outf);


/*** BASIC TABLEAU FUNCTIONS ***/


tableau_t *tableau_copy(tableau_t *tableau);
tableau_t *tableau_create_empty(int sides, int weight);
tableau_t *tableau_create_filled(int sides, int weight, int *row1, int *row2);
void tableau_free(tableau_t *tableau);
int tableau_is_standard(tableau_t *tableau);
int tableau_is_valid(tableau_t *tableau);


/*** STANDARDIZATION FUNCTIONS ***/


sum_t *tableau_standardize(db_t *db, tableau_t *tableau);
int tableau_reorder(tableau_t *tableau);
int tableau_sort_cols(tableau_t *tableau);


/*** LOOP OVER ALL STANDARD TABLEAU ***/


int standard_add_all(db_t *db, int weight, int wantlookup);


/*** ADD MATCHINGS ***/

void add_matchings(db_t *db, int sides);
void add_squared_matchings(db_t *db, int sides);

#endif

